@@ -264,6 +264,32 @@ impl Slack {
         Ok(r.billable_info)
     }
 
+    /// Open a view.
+    /// FROM: https://api.slack.com/methods/views.open
+    pub async fn open_view(&self, body: &View) -> Result<Modal> {
+        // Build the request.
+        let request = self.request(&self.token, Method::POST, "views.open", body, None)?;
+
+        let resp = self.client.execute(request).await?;
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                bail!("status code: {}, body: {}", s, resp.text().await?);
+            }
+        };
+
+        let r: ViewResponse = resp.json().await?;
+
+        if !r.ok {
+            bail!(
+                "status code: {}, body: {}",
+                StatusCode::OK,
+                serde_json::json!(r).to_string()
+            );
+        }
+        Ok(r.view)
+    }
+
     /// Get channel id from name.
     pub async fn channel_id(&self, name: &str) -> Result<String> {
         let channels = self.list_channels().await?;
@@ -548,6 +574,43 @@ pub struct BotCommand {
     pub user_id: String,
 }
 
+#[derive(Debug, Clone, Deserialize, JsonSchema, Serialize)]
+pub struct View {
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub trigger_id: String,
+    #[serde(default)]
+    pub view: Modal,
+}
+
+#[derive(Debug, Clone, Deserialize, JsonSchema, Serialize)]
+pub struct ViewResponse {
+    #[serde(default)]
+    pub ok: bool,
+    #[serde(default)]
+    pub view: Modal,
+    #[serde(default)]
+    pub response_metadata: serde_json::Value,
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub error: String,
+}
+
+/// A modal to send to Slack.
+#[derive(Debug, Clone, Default, Deserialize, JsonSchema, Serialize)]
+pub struct Modal {
+    #[serde(default, rename = "type")]
+    pub type_: ModalType,
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub callback_id: String,
+    #[serde(default)]
+    pub title: MessageBlockText,
+    #[serde(default)]
+    pub submit: MessageBlockText,
+    #[serde(default)]
+    pub close: MessageBlockText,
+    #[serde(default, skip_serializing_if = "Vec::is_empty")]
+    pub blocks: Vec<InputBlock>,
+}
+
 /// A formatted message to send to Slack.
 ///
 /// Docs: https://api.slack.com/messaging/composing/layouts
@@ -714,6 +777,8 @@ pub enum MessageBlockType {
     Divider,
     #[serde(rename = "actions")]
     Actions,
+    #[serde(rename = "input")]
+    Input,
 }
 
 impl Default for MessageBlockType {
@@ -722,7 +787,20 @@ impl Default for MessageBlockType {
     }
 }
 
-/// Action block in Slack.
+/// A modal type in Slack.
+#[derive(Debug, Clone, Deserialize, JsonSchema, Serialize)]
+pub enum ModalType {
+    #[serde(rename = "modal")]
+    Modal,
+}
+
+impl Default for ModalType {
+    fn default() -> Self {
+        ModalType::Modal
+    }
+}
+
+/// Block options in Slack.
 #[derive(Debug, Clone, JsonSchema, Deserialize, Serialize)]
 #[serde(untagged)]
 pub enum BlockOption {
@@ -739,6 +817,44 @@ pub struct MessageBlockText {
     pub text: String,
 }
 
+/// Input block in Slack.
+#[derive(Debug, Clone, Default, JsonSchema, Deserialize, Serialize)]
+pub struct InputBlock {
+    #[serde(rename = "type")]
+    pub type_: MessageBlockType,
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub element: Option<InputBlockElement>,
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub label: Option<MessageBlockText>,
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub text: Option<MessageBlockText>,
+}
+
+/// Input block element in Slack.
+#[derive(Debug, Clone, Default, JsonSchema, Deserialize, Serialize)]
+pub struct InputBlockElement {
+    #[serde(rename = "type")]
+    pub type_: InputType,
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub action_id: String,
+
+    // These two only apply to static select.
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub placeholder: Option<MessageBlockText>,
+    #[serde(default, skip_serializing_if = "Vec::is_empty")]
+    pub options: Vec<SelectInputOption>,
+}
+
+/// Select input option in Slack.
+#[derive(Debug, Clone, Default, JsonSchema, Deserialize, Serialize)]
+pub struct SelectInputOption {
+    #[serde(default)]
+    pub text: MessageBlockText,
+
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub value: String,
+}
+
 /// Action block in Slack.
 #[derive(Debug, Clone, Default, JsonSchema, Deserialize, Serialize)]
 pub struct ActionBlock {
@@ -772,6 +888,21 @@ impl Default for MessageType {
     }
 }
 
+/// Input type in Slack.
+#[derive(Debug, Clone, Deserialize, JsonSchema, Serialize)]
+pub enum InputType {
+    #[serde(rename = "static_select")]
+    StaticSelect,
+    #[serde(rename = "plain_text_input")]
+    PlainText,
+}
+
+impl Default for InputType {
+    fn default() -> Self {
+        InputType::PlainText
+    }
+}
+
 /// Message block accessory in Slack.
 #[derive(Debug, Clone, Default, JsonSchema, Deserialize, Serialize)]
 pub struct MessageBlockAccessory {
@@ -1096,6 +1227,8 @@ pub struct InteractivePayload {
     #[serde(default)]
     pub container: Container,
     #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub callback_id: String,
+    #[serde(default, skip_serializing_if = "String::is_empty")]
     pub trigger_id: String,
     #[serde(default)]
     pub team: Team,
